var RUBIK = {};

var audio = {};
var hasLocalStorage = false;
var muted = false;

function toggleSound() {
	muted = !muted;
	if (hasLocalStorage) {
		try {
			localStorage.setItem('muted', muted.toString());
		} catch (e) {}
	}
	updateSoundIcon();
}

function updateSoundIcon() {
	if(muted) {
		$('.sound').addClass('muted');
	} else {
		$('.sound').removeClass('muted');
	}
}

function loadAudio(name) {
	var xhr = new XMLHttpRequest();
	xhr.open('GET', '../game/assets/audio/' + name + ".mp3", true);
	xhr.responseType = 'arraybuffer';
	xhr.onload = function() {
		context.decodeAudioData(xhr.response, function(buffer) {
			audio[name] = buffer;
		}, function() {});
	};
	xhr.send();
}

function playAudio(name) {
	console.log("PlayA " + name,muted,audio);
	if (muted || !audio[name]) {
		return;
	}
	console.log("PlayB");
	if (context && context.resume) {
		context.resume();
	}
	console.log("PlayC");
	var source = context.createBufferSource();
	source.buffer = audio[name];
	source.connect(context.destination);
	if (source.start) {
		console.log("Start");
		source.start(0);
	} else {
		console.log("NoteOn");
		source.noteOn(0);
	}
	console.log("out");
}

(function($){
	//Load the necessary js files here instead of from content.html because the scripts could depend on js loaded before the footer in the template
	//Make sure each script has its sha-256 added to manifest

	//Load "top level" scripts (it's OK if these rely on jQuery since it's already finished loading)
	$.getScript(chrome.runtime.getURL("game/libs/tween.r12.min.js"));//'sha256-QiwBQCx6qvm4L2ioncuUoClRoE8/39lYi2QAZN1NrnU='
	$.getScript(chrome.runtime.getURL("game/libs/deviceMotion.min.js"));//'sha256-quHyHlkD7nXbxLqyYZ8AuOvPPHqVO7Sqqi+0/AZAjjo='
	$.getScript(chrome.runtime.getURL("game/libs/frames.min.js"));//'sha256-yv6eyBLn0gK381ohNLkxIfOY+BY9OfLk8CMfVjjlOSs='

	//Load Three.js and then all of its dependencies. Notice that we must wait to load dependencies until the original JS is fully loaded
	$.getScript(chrome.runtime.getURL("game/libs/Three.r66.noStrict.min.js"), function() {//'sha256-DXEqUOpamA3rpr9Nojt0+VQM1NXaYjPyXxFA2h88W/U='
		//Load scripts that require Three.js
		$.getScript(chrome.runtime.getURL("game/libs/cuber.min.js"), function() {//'sha256-+zRkYGU9h1iMbTAgIJbJFsx2bZ3wPgZO4tD8x+27KI8='
			//Load scripts that require Cuber.js (because of ERNO)
			$.getScript(chrome.runtime.getURL("game/libs/iecss3d.min.js"));//'sha256-9pbDdgZ7KXTqcNqZ9Uq484mFldf56AXdiwDXF7YVtNI='
			$.getScript(chrome.runtime.getURL("game/libs/locked.min.js"));//'sha256-LP7msr4oZepFOGdy4bkhHtmsdfiYzi6NLvCBFqyLsQo='
			$.getScript(chrome.runtime.getURL("game/libs/certificate.min.js"), function() {//'sha256-NZfB4uoKguySldmCQGDPwHnb2ZR49PD71dNECl5kP+E='
				//Load scripts that require Certificate.js (because of dataStore)
				//TODO $.getScript(chrome.runtime.getURL("game/libs/main3.min.js"));//'sha256-Q4JNsoR2yy3vx3MdgfiTgqGWUKPQB6i4+fBZJ/b9CVU='
				init();
				initAudio();
			});
		});
		$.getScript(chrome.runtime.getURL("game/libs/ierenderer.min.js"));//'sha256-gxs+2ROED6jzus97W3NukOLI3igEpmyWck5WX7GOGvU='
		$.getScript(chrome.runtime.getURL("game/libs/textBox.min.js"));//-'sha256-GCeIiruJW74Qw43pa4WyUjmM3cEY0kz3NwvQ1LdXnTc='
		$.getScript(chrome.runtime.getURL("game/libs/ResizeableTextBox.3.min.js"));//'sha256-3lol8vXms+LvjjmOatweKNa3TDcYC5C2Iw0LccGOtNQ='
		$.getScript(chrome.runtime.getURL("game/libs/divBox.min.js"));//-'sha256-rT/l/CTIC9MIeTHVRDDBQBPEWdJhgc3mywQ0ZgDYwzE='
	});
	
	function init() {

		//Prevent context menu from showing on right click becuase it's used for rotating the cube
		window.oncontextmenu = function ()
		{
			return false;
		}
			
		//Initialize the domain object
		initRubik();

		//Listen to wheel events for rotating the cube
		window.addEventListener('wheel', function(event){
			RUBIK.onWheelRotate(event);
		});

		//Listen to arrow key events for rotating the cube
		$("body").keydown(function(event){
			switch (event.key) {
				case 'ArrowUp':
					RUBIK.rotateCubeUp();
					break;
				case 'ArrowDown':
					RUBIK.rotateCubeDown();
					break;
				case 'ArrowLeft':
					RUBIK.rotateCubeLeft();
					break;
				case 'ArrowRight':
					RUBIK.rotateCubeRight();
					break;
			}
		});

		//Hide the bottons until the animations are complete so the user can't break them
		RUBIK.hideFooterButtons();

		//Initialize cube parameters and add it to the board
		RUBIK.initCube();
		
		RUBIK.initSolveCounter();
		RUBIK.initMovesIgnored();
		RUBIK.initMovesSequence();

		//Listen to clicks on the cube
		$(cube).click({event}, RUBIK.onClickCube);
		
		//Listen for a request to reset the game
		$(RUBIK.reseter).click(RUBIK.resetMovesHistory);

		//Listen for a request to shuffle
		$(RUBIK.shuffler).click(RUBIK.shuffle);

		//Listen for a request to solve the cube
		$(RUBIK.solver).click(RUBIK.solveCube);

		//Listen for a request to stop showing the win-lose screen
		$('body').on('click', RUBIK.hideSolvedScreen, function(){
			RUBIK.hideSolved();
		});
	}

	function initAudio(){
		//Determine if local storage is available		
		try {
			hasLocalStorage = 'localStorage' in window && window['localStorage'] !== null;
		} catch (e) {
			hasLocalStorage = false;
		}
	
		//Initialize saved preferences
		//TODO eventually save these to Google's shared prefs. We'll need "permissions": ["storage"]
		if (hasLocalStorage) {
			try {
				muted = localStorage.getItem('muted') === 'true';
			} catch (e) {
			}
		}
		updateSoundIcon();
	
		//Initialize audio
		window.AudioContext = window.AudioContext || window.webkitAudioContext;
		if (window.AudioContext) {
			context = new AudioContext();
			loadAudio('move');
			loadAudio('solve');
		} else {
			$('.sound').addClass('hide');
		}

		$('.sound').on('click', function(event) {
			event.preventDefault();				
			toggleSound();
		});

		//Animate header svgs on hover
		$('.sound, .solve').hover(
			function(){$(this).addClass('pound')},
			function(){ $(this).removeClass('pound')}
		);
	}

	function initRubik() {
		RUBIK.motion;
		RUBIK.startTime = 0;
		RUBIK.ua = navigator.userAgent;
		RUBIK.isIe = RUBIK.ua.indexOf('MSIE') > -1 || RUBIK.ua.indexOf('Trident') > -1;
		RUBIK.useLockedControls = true;
		RUBIK.controls = RUBIK.useLockedControls ? ERNO.Locked : ERNO.Freeform;
		RUBIK.savedSequence = '';
		RUBIK.SAVEDSEQ_INIT;
		RUBIK.doShowLogos = true;
		RUBIK.isSolving = false;
		RUBIK.isTwisting = true;
		RUBIK.movesIgnored = 0;

		//UI elements - easier to change if they're all in one place
		RUBIK.container = '.board';
		RUBIK.movesCounter = '.score.moves';
		RUBIK.solvedCounter = '.score.solved';
		RUBIK.reseter = '#reset';
		RUBIK.shuffler = '#shuffle';
		RUBIK.solver = '.solve';
		RUBIK.footerButtons = '.footer ul li';
		RUBIK.winnerLoser = '#win-lose';
		RUBIK.hideSolvedScreen = '#hide-solved';

		//Cube history
		RUBIK.defaultSequence = '';
		RUBIK.animationTwistDuration = 500;
		RUBIK.userTwistDuration = 1000;
		RUBIK.ANIMATE_ALL = -1;		
		RUBIK.animateSequenceCount = 0;//0 setting this to zero just shows the cube as it was last time you closed the popup
		RUBIK.includeRotationsInHistory = false;//Whether or not to include cube rotations in the history, and hence count as a move
	}

	//Initialize cube with desired parameters and then add the cube to the board
	RUBIK.initCube = function() {
			
		//Create the cube
		window.cube = new ERNO.Cube({
			hideInvisibleFaces: false,
			controls: RUBIK.controls,
			renderer: RUBIK.isIe ? ERNO.renderers.IeCSS3D : null
		});

		//Setup cube for freeflow or snap
		if (RUBIK.controls === ERNO.Locked){
			var fixedOrientation = new THREE.Euler(Math.PI * 0.1, Math.PI * -0.25, 0);
			cube.object3D.lookAt(cube.camera.position);
			cube.rotation.x += fixedOrientation.x;
			cube.rotation.y += fixedOrientation.y;
			cube.rotation.z += fixedOrientation.z;
		}

		//Setup the camera - this determines how big/small the cube is and the angle from which the user sees it
		cube.camera.position.z = 2100;//TODO - make this a user setting to zoom in and out of the cube
		cube.camera.fov = 20;
		cube.camera.updateProjectionMatrix();

		//Not 100% sure, but I feel like this need to be set at least once or, ocassionally, the cube colors aren't set
		RUBIK.setCubeColors('#fff', '#fff', '#fff', '#fff', '#fff', '#fff', '#fff');

		//Pull the cube down a little so it doesn't overlap too much with the header
		RUBIK.setCubePosition(cube.position.x, -30, cube.position.z);

		//Show the cube from an isometric angle
		RUBIK.setCubeAngle(25, -45, 0);

		//Init user input
		cube.mouseControlsEnabled = false;
		cube.keyboardControlsEnabled = false;
		
		cube.twistCountDown = 0;

		//Init audio
		cube.audioList = [];
		cube.audio = 0;

		//Add the cube to the board
		$(RUBIK.container).append(cube.domElement);

		cube.show();
	}

	//Determine whether how many times the user has solved the cube and display it
	RUBIK.initSolveCounter = function() {
		chrome.storage.sync.get(['rubiksSolved'], function(result) {
			$(RUBIK.solvedCounter).text(result.rubiksSolved);
		});
	}

	RUBIK.initMovesIgnored = function() {
		chrome.storage.sync.get(['movesIgnored'], function(result) {
			if (result.movesIgnored == null || result.movesIgnored == ''){
				//If null, undefined, or empty then set up the default value...
				RUBIK.movesIgnored = 0;
				
				//...and save it for next time
				chrome.storage.sync.set({'movesIgnored':RUBIK.movesIgnored});
			}else{
				RUBIK.movesIgnored = result.movesIgnored;
			}		 
		});
	}

	RUBIK.initMovesSequence = function() {
		chrome.storage.sync.get(['sequenceToRestore'], function(result) {
			
			if (result.sequenceToRestore == null || result.sequenceToRestore == ''){
				//If null, undefined, or empty then set up the default value...
				RUBIK.savedSequence = RUBIK.defaultSequence;
				
				//...and save it for next time
				chrome.storage.sync.set({'sequenceToRestore':RUBIK.savedSequence});
			}else{
				RUBIK.savedSequence = result.sequenceToRestore;
			}
			
			//Restore the user's history by applying it to the cube. This can be animated, not animated, or a combo set by RUBIK.animateSequenceCount
			//There is no need to reset RUBIK.savedSequence here because the applicattion of the non-animated or animated history do not save the move;
			//only the intentional move from user input updates the history.
			//Also, No need to update the "database" because RUBIK.savedSequence is updated and then saved over the current value each time onTwistCompleted is called
			RUBIK.getSavedSequenceSplit();
			RUBIK.finalizeCube();		 
		});
	}
	
	//Split the saved sequence into the last ten moves and all the rest. This is used to only show the first ten elements while still keeping
	//the remaining moves in the moves history.
	RUBIK.getSavedSequenceSplit = function() {

		if (RUBIK.savedSequence == null || RUBIK.savedSequence == ''){
			RUBIK.SAVEDSEQ_INIT = ['',''];
		
		}else if(RUBIK.animateSequenceCount == 0){
			//Don't animate anything
			RUBIK.SAVEDSEQ_INIT = [RUBIK.savedSequence, ''];

		}else if(RUBIK.animateSequenceCount == RUBIK.ANIMATE_ALL || RUBIK.savedSequence.length < RUBIK.animateSequenceCount){
			//If the user wants to show the entire history or the sequence is small enough, animate it all
			//Animate every move from the beginning of time
			RUBIK.SAVEDSEQ_INIT = ['', RUBIK.savedSequence];

		}else{
			//The sequence is too large and we indicated that we didn't want to animate it all. So, just show the most recent moves
			RUBIK.SAVEDSEQ_INIT = [
				RUBIK.savedSequence.substring(0, RUBIK.savedSequence.length - RUBIK.animateSequenceCount),
				RUBIK.savedSequence.substring(RUBIK.savedSequence.length - RUBIK.animateSequenceCount)
			]
		}
	}

	RUBIK.finalizeCube = function() {
				  
		var nonAnimatedMoves = RUBIK.SAVEDSEQ_INIT[0];
		var animatedMoves = RUBIK.SAVEDSEQ_INIT[1];

		cube.twistCountDown = nonAnimatedMoves.length;
		
		if (nonAnimatedMoves.length > 0){
			//There are non-animated moves that need to be applied before animated moves...
			cube.addEventListener('onTwistComplete', RUBIK.onNonAnimatedTwistComplete);

			//Rotate the first bit of the sequence without showing it 
			cube.twistDuration = 0;
			cube.twist(nonAnimatedMoves);

		}else if(animatedMoves.length > 0){
			//There are no non-animated moves to apply but there are still animated moves that need to be applied...
			RUBIK.scrambleCube();

		}else{
			//Allow the user to start manipulating the cube
			RUBIK.startInteractiveCube();
		}
	}

	RUBIK.onNonAnimatedTwistComplete = function() {		
		RUBIK.updateMovesCounter();

		if (cube.twistQueue.history.length - cube.twistCountDown == 0) {
			cube.removeEventListener('onTwistComplete', RUBIK.onNonAnimatedTwistComplete);
			RUBIK.scrambleCube();
		}
	}

	RUBIK.onAnimatedTwistComplete = function() {
		RUBIK.updateMovesCounter();

		//Doing this here just because sometimes it doesn't work and we get an all white cube :(
		//This is fine given that each move will be visible and hence we won't get the disco ball effect
		RUBIK.showCubeFaces();

		if (cube.twistQueue.history.length - cube.twistCountDown == 0) {
			cube.removeEventListener('onTwistComplete', RUBIK.onAnimatedTwistComplete);
			RUBIK.startInteractiveCube();
		}
	}

	RUBIK.onPlayerTwistComplete = function(event){
		//Chris ... this is a good place for code that relies on a twist of the cube/row/column and not a slight tween back into the previous position 
		
		if (cube.isSolved() && !RUBIK.isSolving && RUBIK.savedSequence.length > 0) {

			console.log("The cube is solved! Need to reset the database for next time");
			
			RUBIK.resetMovesHistory();//Reset here just in case they close the popup without clicking reset
			RUBIK.updateSolvedCount();
			RUBIK.showSolved();
			playAudio('jump');
		}else{
			RUBIK.updateMoves(event);
			RUBIK.isTwisting = false;
		}
	}

	RUBIK.onResetTwistComplete = function(event){
		if (cube.isSolved()) {
			//For whatever reason, isSolved is the only method that works. The following doesn't work, so do try it again:
			//cube.twistQueue.history.length == 0 && cube.undoing == !0

			cube.twistDuration = RUBIK.userTwistDuration;
		
			//Reset our tracked history - do it after all moves are undone in the cube or it gets out of sync
			RUBIK.savedSequence = '';
			chrome.storage.sync.set({'sequenceToRestore':RUBIK.savedSequence});
			RUBIK.movesIgnored = 0;
			chrome.storage.sync.set({'movesIgnored':0});

			//Update the counter
			RUBIK.updateMovesCounter();

			//Enable input
			cube.mouseControlsEnabled = true;
			cube.keyboardControlsEnabled = true;
			RUBIK.showFooterButtons();

			cube.removeEventListener('onTwistComplete', RUBIK.onResetTwistComplete);
			cube.addEventListener('onTwistComplete', RUBIK.onPlayerTwistComplete);

			RUBIK.isSolving = false;
		}
	}

	RUBIK.onShuffledTwistComplete = function(event) {
		//Don't count shuffled moves against the user. Indeed, reset to zero even if user had previous moves.
		//Need to add 1 since we ignore and then call update moves (which calles updateMovesCounter)
		RUBIK.movesIgnored = RUBIK.savedSequence.length + 1;

		//Save move history
		RUBIK.updateMoves(event);

		//There is no reason to update the moves history here since it's already done with the onPlayerTwistComplete listener
		if (cube.twistQueue.history.length - cube.twistCountDown == 0) {
			cube.removeEventListener('onTwistComplete', RUBIK.onShuffledTwistComplete);
			RUBIK.startInteractiveCube();
		}
	}

	//Animate the most recent moves if desired
	RUBIK.scrambleCube = function() {
		//Colorize the faces now that the history has been applied
		RUBIK.showCubeFaces();
		
		var animatedMoves = RUBIK.SAVEDSEQ_INIT[1];
		
		if (animatedMoves.length > 0){
			//Listen for onTwistComplete
			cube.addEventListener('onTwistComplete', RUBIK.onAnimatedTwistComplete);

			//Animate the desired history
			cube.twistDuration = RUBIK.animationTwistDuration;
			cube.twistCountDown = animatedMoves.length + cube.twistQueue.history.length;
			cube.twist(animatedMoves);
		} else {
			//There was no history to animate, so, just allow the user to start manipulating the cube
			RUBIK.startInteractiveCube();
		}
	}

	RUBIK.startInteractiveCube = function() {
				
		//For whatever reason, the cube faces are not always applied. So, we try quickly and force it with a little timer
		RUBIK.showCubeFaces();
		window.setTimeout(function() {
			RUBIK.showCubeFaces();
		},100);

		//Now that everything has stopped animating, allow the user to interact with the buttons again
		RUBIK.showFooterButtons();

		RUBIK.enableDeviceMotion();

		// var sentCertificate = false;
		RUBIK.startTime = (new Date()).getTime();
		cube.twistDuration = RUBIK.userTwistDuration;
		cube.moveCounter = 0;//TODO why is this any different than our own moveCounter?

		cube.addEventListener('onTwistComplete', RUBIK.onPlayerTwistComplete);

		cube.mouseControlsEnabled = true;
		cube.keyboardControlsEnabled = true;
		RUBIK.isTwisting = false;
	}

	RUBIK.enableDeviceMotion = function () {
		if (!RUBIK.motion) {
			RUBIK.motion = deviceMotion(cube, $(RUBIK.container)[0]);
			RUBIK.motion.decay = 0.1;
			RUBIK.motion.range.x = Math.PI * 0.02;
			RUBIK.motion.range.y = Math.PI * 0.02;
			RUBIK.motion.range.z = 0;
		}
		RUBIK.motion.paused = false;
	}

	RUBIK.updateMoves = function(event){
		RUBIK.showCubeFaces();

		//TODO only play sounds after initialization because this is called many times at once and it creates an awful sound

		var twist = event.detail.twist;
		var cmd = twist.command;
		
		//Determine if the cube is being rotated in order to view it from a different angle
		var rotatingCube = cmd == "x" || cmd == "y" || cmd == "z" || cmd == "X" || cmd == "Y" || cmd == "Z" ? true : false;	
		if (rotatingCube){
			//Play rotation sounds
			playAudio('move');

			if(!RUBIK.includeRotationsInHistory){
				//Don't count the rotation as a move - this means don't add to history or increment move count.
				return;
			}
		}

		//TODO handle undo. IMHO, we should remove the item and the decrement the count because an undo should restore the previous state.
		//Looking at cube.moveCounter in Cuber.js, seems like they agree

		//When not moving at least 90 degrees then ignore the twist; if moving more than 90, it means multiple turns, so record multiple twists,
		//otherwise the saved state gets completely out of sync with the current state
		var degs = twist.degrees == null ? 90 : twist.degrees;//tapping a row or column results in twist.degrees=undefined; but we know it is one rotation
		var rotations = degs/90;
		for(i = 0; i < rotations; i++){
			RUBIK.savedSequence = RUBIK.savedSequence + cmd;
			
			if(!rotatingCube){
				//Only play the twisting sound since the rotation sound would have already been played above
				playAudio('move');
			}
		}

		//Add the new move to the history and save the updated history
		chrome.storage.sync.set({'sequenceToRestore':RUBIK.savedSequence});
		chrome.storage.sync.set({'movesIgnored':RUBIK.movesIgnored});
	
		RUBIK.updateMovesCounter();
	}

	RUBIK.updateMovesCounter = function() {
		if (RUBIK.movesCounter) {
			var moves = RUBIK.savedSequence.length - RUBIK.movesIgnored;				
			$(RUBIK.movesCounter).html(moves);
		}
		RUBIK.updateResetter();
	}

	RUBIK.hideFooterButtons = function() {
		$(RUBIK.footerButtons).addClass('hide');
	}

	RUBIK.showFooterButtons = function() {
		$(RUBIK.footerButtons).removeClass('hide');
		RUBIK.updateResetter();
	}

	RUBIK.updateResetter = function() {
		if(RUBIK.savedSequence.length == 0){
			$(RUBIK.reseter).hide();
		}else{
			$(RUBIK.reseter).show();
		}
	}

	RUBIK.onClickCube = function(event){
		//Use the window's event to distinguish between right, center and left because the event passed here is from cuber.js
		//and doesn't easily allow us to determine left/right
		var isRightClick = false;
    	var winClickEvent = window.event;

    	if ("which" in winClickEvent){  // Gecko (Firefox), WebKit (Safari/Chrome) & Opera
			isRightClick = winClickEvent.which == 3; 
			
		}else if ("button" in winClickEvent){  // IE, Opera 
			isRightClick = winClickEvent.button == 2; 
		}

		if (isRightClick)
		{
			RUBIK.rotateCubeRight();
		}else{
			RUBIK.rotateCubeLeft();
		}
	}

	RUBIK.onWheelRotate = function(event){
		
		if (event.deltaY < 0)
		{
			RUBIK.rotateCubeUp();
		}
		else if (event.deltaY > 0)
		{
			RUBIK.rotateCubeDown();
		}
	}

	RUBIK.rotateCubeLeft = function(){
		//Ignore clicks if mouse controls disabled
		if (!cube.mouseControlsEnabled || RUBIK.isTwisting) {
			return;
		}

		//Rotate cube left
		RUBIK.isTwisting = true;
		cube.twist("Y");
	}

	RUBIK.rotateCubeRight = function(){
		//Ignore clicks if mouse controls disabled
		if (!cube.mouseControlsEnabled || RUBIK.isTwisting) {
			return;
		}

		//Rotate cube right
		RUBIK.isTwisting = true;
		cube.twist("y");
	}

	RUBIK.rotateCubeUp = function(){
		//Ignore clicks if mouse controls disabled
		if (!cube.mouseControlsEnabled || RUBIK.isTwisting) {
			return;
		}

		//Rotate cube up
		RUBIK.isTwisting = true;
		cube.twist("z");
	}

	RUBIK.rotateCubeDown = function(){
		//Ignore clicks if mouse controls disabled
		if (!cube.mouseControlsEnabled || RUBIK.isTwisting) {
			return;
		}

		//Rotate cube down. 
		RUBIK.isTwisting = true;
		cube.twist("Z");
	}

	RUBIK.clickToRotateRows = function(event){

		//Ignore clicks if mouse controls disabled
		if (!cube.mouseControlsEnabled || RUBIK.isTwisting) {
			return;
		}
		RUBIK.isTwisting = true;
		  
  		var cubelet = event.detail.cubelet,
      		face = cubelet[event.detail.face.toLowerCase()],
      		axis = new THREE.Vector3(),
      		exclude = new THREE.Vector3(1, 0, 0),
      		UP = new THREE.Vector3(0, 1, 0),
			normal = ERNO.Direction.getDirectionByName(face.normal).normal.clone(), 
			slice;
			  
  		normal.x = Math.abs(normal.x);
  		normal.y = Math.abs(normal.y);
		normal.z = Math.abs(normal.z);
		  
  		var l = cube.slices.length;
 		while (l-- > 0) {
			slice = cube.slices[l];
    		axis.copy(slice.axis);
    		axis.x = Math.abs(axis.x);
    		axis.y = Math.abs(axis.y);
			axis.z = Math.abs(axis.z);
			
			if (slice.cubelets.indexOf(cubelet) !== -1 && axis.equals(UP)) {
      			break;
    		}
		  }
		  
  		var command = slice.name.substring(0, 1);
  		if (slice === cube.down){
			command = command.invert();
		}

  		//Twists the cube by the desired amount
  		cube.twist(command);
	}

	RUBIK.animateToCubePosition = function(xF, yF, zF, duration){
		new TWEEN.Tween(cube.position)
		.to({
				x: xF,
				y: yF,
				z: zF
			}, duration)
		.easing(TWEEN.Easing.Quartic.InOut)
		.start(cube.time);
	}

	RUBIK.animateToCubeAngle = function(degX, degY, degZ, duration){
		new TWEEN.Tween(cube.rotation)
		.to({
				x: (degX).degreesToRadians(),
				y: (degY).degreesToRadians(),
				z: (degZ).degreesToRadians()
			}, duration) 
		.easing(TWEEN.Easing.Quartic.InOut)
		.start(cube.time);
	}

	RUBIK.hideCube = function() {
		cube.hide();
	}

	RUBIK.showCube = function() {
		cube.show();
	}

	RUBIK.setCubeSize = function(width, height){
			cube.setSize(width, height);
	}

	//Ex. '#DC422F', '#fff', '#3D81F6', '#009D54', '#FF6C00', '#FDCC09', '#cdcdcd'
	RUBIK.setCubeColors = function(color1, color2, color3, color4, color5, color6, cubeColor){
		ERNO.RED.hex = color1;
		ERNO.WHITE.hex = color2;
		ERNO.BLUE.hex = color3;
		ERNO.GREEN.hex = color4;
		ERNO.ORANGE.hex = color5;
		ERNO.YELLOW.hex = color6;
		ERNO.COLORLESS.hex = cubeColor;
	}
	
	RUBIK.setCameraFieldOfView = function(fov){
		cube.camera.fov = fov;
		cube.camera.updateProjectionMatrix();
	}

	RUBIK.setCameraDepth = function(depth){
		cube.camera.position.z = depth;
		cube.camera.updateProjectionMatrix();
	}

	RUBIK.setCameraSettings = function(fov, depth){
		cube.camera.fov = fov;
		cube.camera.position.z = depth;
		//TODO - other camera settings
		cube.camera.updateProjectionMatrix();
	}

	RUBIK.setCubePosition = function(fromLeft, fromBottom, depth){
		cube.position.x = fromLeft;
		cube.position.y = fromBottom;
		cube.position.z = depth;
	}

	RUBIK.setCubeAngle = function(degX, degY, degZ){
		cube.rotation.set(
			(degX).degreesToRadians(),
			(degY).degreesToRadians(),
			(degZ).degreesToRadians()
		);
	}

	RUBIK.hideLogos = function() {
		$('.stickerLogo').removeClass('tCubedLogo');
	}

	RUBIK.showLogos = function() {
		$('.stickerLogo').addClass('tCubedLogo');
	}

	RUBIK.hideCubeFaces = function() {
		$('.sticker.red').removeClass('color1');
		$('.sticker.white').removeClass('color2');
		$('.sticker.blue').removeClass('color3');
		$('.sticker.green').removeClass('color4');
		$('.sticker.orange').removeClass('color5');
		$('.sticker.yellow').removeClass('color6');

		RUBIK.hideLogos();
	}

	RUBIK.showCubeFaces = function() {
		$('.sticker.red').addClass('color1');
		$('.sticker.white').addClass('color2');
		$('.sticker.blue').addClass('color3');
		$('.sticker.green').addClass('color4');
		$('.sticker.orange').addClass('color5');
		$('.sticker.yellow').addClass('color6');

		if (RUBIK.doShowLogos){
			RUBIK.showLogos();
		}else{
			RUBIK.hideLogos();
		}
	}

	RUBIK.updateSolvedCount = function() {
		//If the user genuinely solved the puzzle, ie did not rely on the solve button, then update the saved value and update the counter
		chrome.storage.sync.get(['rubiksSolved'], function(result) {
			var solveCnt = result.rubiksSolved != null ? result.rubiksSolved + 1 : 1;			 
			chrome.storage.sync.set({'rubiksSolved':solveCnt});
			$(RUBIK.solvedCounter).html(solveCnt);
		});
	}

	RUBIK.shuffle = function() {
		//Disable input
		cube.mouseControlsEnabled = false;
		cube.keyboardControlsEnabled = false;
		RUBIK.isTwisting = true;
		RUBIK.hideFooterButtons();

		//Do not add rotations here! It makes the cube look different even if the matrix is the same
		var shuffles = [
			"lludLrbeDmRMedBDmE",
			"DDDrrMedBEBRlDULLR",
			"LRLUmDudLrBrLrbeDm"
		];

		const theShuffle = shuffles[Math.floor(Math.random() * shuffles.length)];
			
		//Stop counting all previous moves since we're shuffling for a new attempt
		RUBIK.movesIgnored = RUBIK.savedSequence.length;
		RUBIK.updateMovesCounter();

		//Only listen for onTwistComplete for the shuffle
		cube.removeEventListener('onTwistComplete', RUBIK.onPlayerTwistComplete);
		cube.addEventListener('onTwistComplete', RUBIK.onShuffledTwistComplete);

		//Animate the desired history
		cube.twistDuration = RUBIK.animationTwistDuration;
		cube.twistCountDown = theShuffle.length + cube.twistQueue.history.length;
		cube.twist(theShuffle);
	}

	RUBIK.resetMovesHistory = function() {
		//Undo each move as seen from the cube's history - this history is the actual, exact, history including rotations
		cube.twistDuration = 0;
		RUBIK.solveCube();
	}

	//Animate the undo
	RUBIK.undo = function() {
		cube.undo();
	}

	//Animate the re-do
	RUBIK.redo = function() {
		cube.redo();
	}

	//This will solve the cube using in a total time =cube.twistQueue.history.length * cube.twistDuration
	//ie to solve at a watchable pace, set cube.twistDuration to something like 500ms. To instantly solve (ie reset) then cube.twistDuration should be 0.
	RUBIK.solveCube = function() {
		if (RUBIK.isSolving || RUBIK.savedSequence.length == 0) {
			console.log("Cube is already solved!");
			return;
		}
		RUBIK.isSolving = true;
		RUBIK.isTwisting = true;

		//This method is called for 1) resetting 2) solving. When resetting, duration = 0 was already set; leave it alone
		cube.twistDuration = cube.twistDuration == 0 ? 0 : RUBIK.animationTwistDuration;

		//Disable input
		cube.mouseControlsEnabled = false;
		cube.keyboardControlsEnabled = false;
		RUBIK.hideFooterButtons();

		//Prevent the playerTwistComplete listener from thinking the undo twist are from the player
		cube.removeEventListener('onTwistComplete', RUBIK.onPlayerTwistComplete);

		//Listen for the twists to be complete instead of resetting values here. This is because the undo process takes time and the values do
		//not get reset correctly otherwise.
		cube.addEventListener('onTwistComplete', RUBIK.onResetTwistComplete);

		while (cube.twistQueue.history.length > 0){
			cube.undo();
		}
	}

	RUBIK.showSolved = function() {
		$(RUBIK.winnerLoser).removeClass('gone');
	}

	RUBIK.hideSolved = function() {
		console.log('Bing');
		$(RUBIK.winnerLoser).addClass('gone');
	}

	RUBIK.logSequenceToRestore = function() {
		chrome.storage.sync.get(['sequenceToRestore'], function(result) {
			if (result.sequenceToRestore == null || result.sequenceToRestore == ''){
				console.log("HISTORY: EMPTY");
			}else{
				console.log("HISTORY:" + result.sequenceToRestore);
			}		 
		});
	}

})(jQuery);